@@ -315,9 +315,10 @@ sub launch_server {
          ($handle,$error)=Net::FullAuto::FA_Core::connect_ssh($s);
       }
       if ($error) {
+         my $stderr=$error;
          my $t="aws ec2 terminate-instances --instance-id $inst->{InstanceId}";
          ($hash,$output,$error)=run_aws_cmd($t);
-         next;
+         &exit_on_error($stderr);
       }
       last;
    }
@@ -534,21 +535,21 @@ my $select_an_instance_type=sub {
       }
    }
 #print "DUMP=",Data::Dump::Streamer::Dump(\@sizes)->Out(),"\n";<STDIN>;
-   my $demo_choice=']T[{choose_demo_setup}';
-   $demo_choice=~s/^["]//;
-   $demo_choice=~s/["]$//;
+   my $instruction_set_choice=']T[{choose_is_setup}';
+   $instruction_set_choice=~s/^["]//;
+   $instruction_set_choice=~s/["]$//;
    my $ns=$#sizes+1;
    my $stype='t2.small';
-   my $iset=$Net::FullAuto::ISets->{$demo_choice};
-   my $defaultInstanceType=$Net::FullAuto::ISets->{$demo_choice}->[1].
+   my $iset=$Net::FullAuto::ISets->{$instruction_set_choice};
+   my $defaultInstanceType=$Net::FullAuto::ISets->{$instruction_set_choice}->[1].
                            'defaultInstanceType';
-   $Net::FullAuto::ISets->{selected_iset}=$demo_choice;
+   $Net::FullAuto::ISets->{selected_iset}=$instruction_set_choice;
    eval "\$stype=$defaultInstanceType";
-   my $result=$Net::FullAuto::ISets->{$demo_choice}->[1]."select_".
-              lc($Net::FullAuto::ISets->{$demo_choice}->[2]).
+   my $result=$Net::FullAuto::ISets->{$instruction_set_choice}->[1]."select_".
+              lc($Net::FullAuto::ISets->{$instruction_set_choice}->[2]).
               "_setup";
    eval "\$result=$result";
-   my $is_name=$Net::FullAuto::ISets->{$demo_choice}->[2];
+   my $is_name=$Net::FullAuto::ISets->{$instruction_set_choice}->[2];
    my @sz=grep { !/t2.micro/ } @sizes;
    unshift @sz, grep { /t2.micro/ } @sizes;
    @sizes=@sz;
@@ -567,7 +568,7 @@ my $select_an_instance_type=sub {
 END
    $select_type_banner.=<<END;
    Choose the type of server instance to use for your
-   $demo_choice demo. Note that $stype
+   $instruction_set_choice build. Note that $stype
    has been pre-selected for you. If you wish use this, just press [ENTER],
    otherwise use the [^] and [v] arrow keys to make a different selection.
    Note: There are $ns choices.
@@ -594,9 +595,9 @@ END
 
 my $choose_an_instance_type=sub {
 
-   my $demo_choice=']T[{choose_demo_setup}';
-   $demo_choice=~s/^["]//;
-   $demo_choice=~s/["]$//;
+   my $instruction_set_choice=']T[{choose_is_setup}';
+   $instruction_set_choice=~s/^["]//;
+   $instruction_set_choice=~s/["]$//;
    my $instance_type_banner=<<'END';
     ___         _                      _____
    |_ _|_ _  __| |_ __ _ _ _  __ ___  |_   _|  _ _ __  ___ ___
@@ -605,8 +606,9 @@ my $choose_an_instance_type=sub {
                                             |__/|_|
 
 END
-   $instance_type_banner.="   You have selected the demo: $demo_choice\n";
-   if (-1<index $demo_choice,'Liferay') {
+   $instance_type_banner.=
+      "   You have selected the Instruction Set: $instruction_set_choice\n";
+   if (-1<index $instruction_set_choice,'Liferay') {
       $instance_type_banner.=<<END;
 
    Unfortunately, Free Tier micro servers do not have enough resources to
@@ -616,7 +618,7 @@ END
    presented to you for approval before any costs are incurred.
 
 END
-   } elsif (-1<index $demo_choice,'Chaining') {
+   } elsif (-1<index $instruction_set_choice,'Chaining') {
       $instance_type_banner.=<<END;
 
    Unfortunately, Free Tier micro servers do not have enough resources to
@@ -626,10 +628,10 @@ END
    presented to you for approval before any costs are incurred.
 
 END
-   } elsif (-1<index $demo_choice,'Hadoop') {
+   } elsif (-1<index $instruction_set_choice,'Hadoop') {
       $instance_type_banner.=<<END;
 
-   $demo_choice can be run on a Free Tier micro server, but the performance
+   $instruction_set_choice can be run on a Free Tier micro server, but the performance
    will be poor. Therefore it is recommended you choose at least a 'small'
    instance type. However, Free Tier remains the default choice. Based on the
    choices you make next, a fee summary will be calculated and presented to
@@ -639,7 +641,7 @@ END
    } else {
       $instance_type_banner.=<<END;
 
-   $demo_choice can be run on a Free Tier micro server, but the
+   $instruction_set_choice can be run on a Free Tier micro server, but the
    performance will be poor. Therefore it is recommended you choose at least
    a 'small' instance type. However, Free Tier remains an option. Based on
    the choices you make next, a fee summary will be calculated and presented
@@ -661,7 +663,7 @@ END
 
 my $choose_aws_instances=sub {
 
-   my $demo_choice=']S[';
+   my $instruction_set_choice=']S[';
    my $fa_tag=0;
    if (-r "/tmp/fa_aws_home.txt") {
       open(RD,"/tmp/fa_aws_home.txt");
@@ -744,7 +746,7 @@ my $get_ec2_api=sub {
    $Net::FullAuto::Cloud::fa_amazon::configure_aws1->()
       if (-1<index $output,'configure credentials') ||
       (-1<index $output,'Partial credentials found');
-   my $choose_demo_banner=<<'END';
+   my $choose_is_banner=<<'END';
 
    Amazon AWS EC2 API is Active!
      ___ _
@@ -756,9 +758,9 @@ my $get_ec2_api=sub {
     | || ' \(_-<  _| '_| || / _|  _| / _ \ ' \   \__ \/ -_)  _|
    |___|_||_/__/\__|_|  \_,_\__|\__|_\___/_||_|  |___/\___|\__|
 
-   Below is a selection of demos designed to demonstrate
-   FullAuto's unique ability to automate *any* cloud
-   computing operation. Please choose one:
+   Below is a selection of FullAuto Instruction Sets designed
+   to demonstrate FullAuto's unique ability to automate *any*
+   cloud computing operation. Please choose one:
 
 END
    my $dirtree='';
@@ -767,9 +769,9 @@ END
    } else {
       $dirtree=Net::FullAuto::FA_Core::get_isets('Amazon');
    }
-   my %choose_demo_setup=(
+   my %choose_is_setup=(
 
-      Name => 'choose_demo_setup',
+      Name => 'choose_is_setup',
       Item_1 => {
 
          Text   => ']C[',
@@ -778,10 +780,10 @@ END
 
       },
       Scroll => 4,
-      Banner => $choose_demo_banner,
+      Banner => $choose_is_banner,
 
    );
-   return \%choose_demo_setup;
+   return \%choose_is_setup;
 };
 
 sub assist_user_to_upload_pemfile {
@@ -826,23 +828,30 @@ END
     | |  V | | |  \ V  V / _` | |  _|_|
    |___|   |_|_|   \_/\_/\__,_|_|\__(_)  (for 5 minutes)
 END
-   $wait_banner.=<<END;
+   my $i_will_wait_sub=sub {
 
-   If you can, go ahead and upload the private key mentioned on the
+      my $keyfilename="]I[{'ask_for_keyfile',1}";
+      $keyfilename=~s/^.*\/(.*)(?:[.]pem)*$/$1/;
+      $keyfilename=~s/[.]pem$/$1/;
+      my $pem_file=$keyfilename.'.pem';
+      $pem_file||='<filename>.pem';
+      $wait_banner.=<<END;
+
+   If you can, go ahead and upload \"$pem_file\" mentioned on the
    previous page right now. (If you need to review the instructions
    again, just use the LEFTARROW [<] to navigate back to the previous
    page.)
 
-   The key should be uploaded to the /home/$user directory. When
-   FullAuto detects the key in the /home/$user directory, it will
-   authenticate and proceed to the next page automatically. Otherwise,
-   FullAuto will timeout and gracefully exit in 5 minutes.
+   $pem_file should be uploaded to the /home/$user directory.
+   When you press [ENTER] on this screen, FullAuto will begin scanning
+   for $pem_file in the /home/$user directory. Once
+   $pem_file is detected, Fullauto will authenticate and proceed
+   to the next page automatically. Otherwise, FullAuto will timeout and
+   gracefully exit in 5 minutes.
 
    If you would like to quit and continue later, just press the ESC key.
 
 END
-   my $i_will_wait_sub=sub {
-
       my %i_will_wait=(
 
          Name => 'i_will_wait',
@@ -855,7 +864,7 @@ END
             foreach my $sec (1..300) {
                sleep 1;
                unless (-e $key) {
-                  open(DH,".");
+                  opendir(DH,".");
                   my @pems=();
                   while (my $line=readdir(DH)) {
                      next if $line eq '.';
@@ -866,6 +875,7 @@ END
                         last;
                      }
                   }
+                  close DH;
                   if (-1<$#pems) {
                      if (0==$#pems) {
                         $pem_file=$pems[0];
@@ -883,6 +893,7 @@ END
                }
             }
             if ($gotkey) {
+               `sudo chmod 400 $key`;
                return $key;
             } else {
                Net::FullAuto::FA_Core::cleanup();
@@ -1231,8 +1242,8 @@ sub new_user_amazon {
    You are fully authenticated with FullAuto on Amazon AWS EC2:
    "Amazon Web Services  -  Elastic Compute Cloud".
 
-   The goal now is to demonstrate how FullAuto can fully automate the
-   setup, processing and maintenance of cloud computing on AWS EC2.
+   The objective is to demonstrate how FullAuto can fully automate the
+   setup, processing and maintenance of cloud computing in AWS EC2.
 
    FullAuto will now check the current system to determine if the ec2 API
    is installed and available for use.
