@@ -1587,7 +1587,7 @@ my $chaining_setup_summary=sub {
    $type=~s/"$//;
    my $money=$type;
    $money=~s/^.*-> \$(.*?) +(?:[(].+[)] )*\s*per hour$/$1/;
-   $type=substr($type,0,(index $type,' ->')-3);
+   $type=~s/^(.*?)\s+-[>].*$/$1/;
    my $catalyst="]T[{select_chaining_setup}";
    $catalyst=~s/^"//;
    $catalyst=~s/"$//;
